@@ -29,7 +29,7 @@
 #define N_RULE_GROUP2    120          // max num of two-letter rule chains
 #define N_HASH_DICT     1024
 #define N_CHARSETS        20
-#define N_LETTER_GROUPS   20
+#define N_LETTER_GROUPS   26
 
 
 /* dictionary flags, word 1 */
@@ -56,7 +56,7 @@
 #define FLAG_ALT_TRANS    0x100000  // language specific
 #define FLAG_ALT2_TRANS   0x200000  // language specific
 
-
+#define FLAG_MAX3       0x08000000  // limit to 3 repeats
 #define FLAG_PAUSE1     0x10000000  // shorter prepause
 #define FLAG_TEXTMODE   0x20000000  // word translates to replacement text, not phonemes
 #define BITNUM_FLAG_TEXTMODE    29
@@ -75,8 +75,10 @@
 #define FLAG_VERB_EXT        0x100  /* extend the 'verb follows' */
 #define FLAG_CAPITAL         0x200  /* pronunciation if initial letter is upper case */
 #define FLAG_ALLCAPS         0x400  // only if the word is all capitals
+#define BITNUM_FLAG_ALLCAPS   0x2a
 #define FLAG_ACCENT          0x800  // character name is base-character name + accent name
-
+#define FLAG_HYPHENATED     0x1000  // multiple-words, but needs hyphen between parts 1 and 2
+#define BITNUM_FLAG_HYPHENATED  0x2c
 
 
 // wordflags, flags in source word
@@ -86,15 +88,16 @@
 #define FLAG_HAS_PLURAL    0x4    /* upper-case word with s or 's lower-case ending */
 #define FLAG_PHONEMES      0x8    /* word is phonemes */
 #define FLAG_LAST_WORD     0x10   /* last word in clause */
-//#define FLAG_SPELLING      0x20  // speak the word as individual letters
 #define FLAG_EMBEDDED      0x40   /* word is preceded by embedded commands */
 #define FLAG_HYPHEN        0x80
 #define FLAG_NOSPACE       0x100  // word is not seperated from previous word by a space
 #define FLAG_FIRST_WORD    0x200  // first word in clause
+#define FLAG_FOCUS         0x400   // the focus word of a clause
+#define FLAG_EMPHASIZED    0x800
+#define FLAG_EMPHASIZED2   0xc00  // FLAG_FOCUS | FLAG_EMPHASIZED
 #define FLAG_DONT_SWITCH_TRANSLATOR  0x1000
 #define FLAG_SUFFIX_REMOVED  0x2000
 #define FLAG_HYPHEN_AFTER    0x4000
-#define FLAG_EMPHASIZED    0x8000
 
 #define FLAG_NO_TRACE      0x10000   // passed to TranslateRules() to suppress dictionary lookup printout
 #define FLAG_NO_PREFIX     0x20000
@@ -161,8 +164,10 @@
 // bits 0-7 pause x 10mS, bits 12-14 intonation type,
 // bit 19=sentence, bit 18=clause,  bits 17=voice change
 // bit 16 used to distinguish otherwise identical types
+// bit 20= punctuation character can be inside a word (Armenian)
 #define CLAUSE_BIT_SENTENCE  0x80000
 #define CLAUSE_BIT_VOICE     0x20000
+#define PUNCT_IN_WORD        0x100000
 
 #define CLAUSE_NONE         0 + 0x04000
 #define CLAUSE_PARAGRAPH   70 + 0x80000
@@ -170,7 +175,7 @@
 #define CLAUSE_VOICE        0 + 0x24000
 #define CLAUSE_PERIOD      35 + 0x80000
 #define CLAUSE_COMMA       20 + 0x41000
-#define CLAUSE_SHORTCOMMA  15 + 0x41000
+#define CLAUSE_SHORTCOMMA   5 + 0x41000
 #define CLAUSE_QUESTION    35 + 0x82000
 #define CLAUSE_EXCLAMATION 40 + 0x83000
 #define CLAUSE_COLON       30 + 0x40000
@@ -187,6 +192,8 @@
 #define SAYAS_DIGITS    0x40  // + number of digits
 #define SAYAS_DIGITS1   0xc1
 
+#define CHAR_EMPHASIS   0x0530  // this is an unused character code
+
 // Rule:
 // [4] [match] [1 pre] [2 post] [3 phonemes] 0
 //     match 1 pre 2 post 0     - use common phoneme string
@@ -306,6 +313,8 @@ typedef struct {
 // bit15= Give stress to the first unstressed syllable
 // bit16= Don't diminish consecutive syllables within a word.
 // bit17= "priority" stress reduces other primary stress to "unstressed" not "secondary"
+// bit18= don't lengthen short vowels more than long vowels at end-of-clause
+// bit19=stress on final syllable if it has a long vowel, but previous syllable has a short vowel
 
 	int stress_flags; 
 	int unstressed_wd1; // stress for $u word of 1 syllable
@@ -314,9 +323,10 @@ typedef struct {
 	unsigned char *length_mods;
 	unsigned char *length_mods0;
 
-#define NUM_ROMAN    0x20000
-#define NUM_ROMAN_UC 0x40000
-#define NUM_NOPAUSE  0x80000
+#define NUM_ROMAN        0x20000
+#define NUM_ROMAN_UC     0x40000
+#define NUM_NOPAUSE      0x80000
+#define NUM_ROMAN_AFTER 0x200000
 
 	// bits0-1=which numbers routine to use.
 	// bit2=  thousands separator must be space
@@ -336,13 +346,18 @@ typedef struct {
 	// bit18=Roman numbers only if upper case
 	// bit19=don't add pause after a number
 	// bit20='and' before hundreds
+	// bit21= say "roman" after the number, not before
 	int numbers;
 
+#define NUM2_100000     0x800   // numbers for 100,000 and 10,000,000
+#define NUM2_100000a    0xc00   // numbers for 100,000 and 1,000,000
 	// bits 1-4  use variant form of numbers before thousands,millions,etc.
 	// bit6=(LANG=pl) two forms of plural, M or MA
 	// bit7=(LANG-ru) use MB for 1 thousand, million, etc
-	// bit8=(LANG=sw) special word for 100,000s
+	// bit8=(LANG=cs,sk) two forms of plural, M or MA
 	// bit9=(LANG=rw) say "thousand" and "million" before its number, not after
+   // bit10=(LANG=sw) special word for 100,000 and 1,000,000
+	// bit11=(LANG=hi) special word for 100,000 and 10,000,000
 	int numbers2;
 
 	int max_roman;
@@ -364,6 +379,7 @@ typedef struct {
 	int testing;            // testing options: bit 1= specify stressed syllable in the form:  "outdoor/2"
 	int listx;    // compile *_listx after *list
 	const unsigned int *replace_chars;      // characters to be substitutes
+	const char *ascii_language;  // switch to this language for Latin characters
 } LANGUAGE_OPTIONS;
 
 
@@ -383,18 +399,9 @@ typedef struct {
 #define NUM_SEP_SPACE  0x1000    // allow space as thousands separator (in addition to langopts.thousands_sep)
 #define NUM_DEC_IT     0x2000    // (LANG=it) speak post-decimal-point digits as a combined number not as single digits
 
-class Translator
+struct Translator
 {//=============
-public:
-	Translator();
-	virtual ~Translator();
-	void *TranslateClause(FILE *f_text, const void *vp_input, int *tone, char **voice_change);
-	int TranslateWord(char *word, int next_pause, WORD_TAB *wtab);
-	int LoadDictionary(const char *name, int no_error);
-	virtual void CalcLengths();
-	virtual void CalcPitches(int clause_tone);
-	void GetTranslatedPhonemeString(char *phon_out, int n_phon_out);
-	
+
 	LANGUAGE_OPTIONS langopts;
 	int translator_name;
 	int transpose_offset;
@@ -402,7 +409,8 @@ public:
 	int transpose_min;
 
 	char phon_out[300];
-	char word_phonemes[N_WORD_PHONEMES];    // a word translated into phoneme codes
+	char phonemes_repeat[20];
+	int phonemes_repeat_count;
 
 	unsigned char stress_amps[8];
 	unsigned char stress_amps_r[8];
@@ -425,56 +433,8 @@ public:
 	char *data_dictrules;     // language_1   translation rules file
 	char *data_dictlist;      // language_2   dictionary lookup file
 	char *dict_hashtab[N_HASH_DICT];   // hash table to index dictionary lookup file
+	char *letterGroups[N_LETTER_GROUPS];
 
-
-private:
-	int TranslateWord2(char *word, WORD_TAB *wtab, int pre_pause, int next_pause);
-	int TranslateLetter(char *letter, char *phonemes, int control, int word_length);
-	void SetSpellingStress(char *phonemes, int control, int n_chars);
-	void WriteMnemonic(int *ix, int mnem);
-	void MakePhonemeList(int post_pause, int new_sentence);
-	int SubstitutePhonemes(PHONEME_LIST2 *plist_out);
-
-	int ReadClause(FILE *f_in, char *buf, short *charix, int n_buf);
-	int AnnouncePunctuation(int c1, int c2, char *buf, int ix);
-
-	const char *LookupDict2(const char *word, const char *word2, char *phonetic, unsigned int *flags, int end_flags, WORD_TAB *wtab);
-	const char *LookupSpecial(const char *string, char *text_out);
-	const char *LookupCharName(int c);
-	void LookupLetter(unsigned int letter, int next_byte, char *ph_buf);
-	int LookupLetter2(unsigned int letter, char *ph_buf);
-	void LookupAccentedLetter(unsigned int letter, char *ph_buf);
-	int LookupNum2(int value, int control, char *ph_out);
-	int LookupNum3(int value, char *ph_out, int suppress_null, int thousandplex, int prev_thousands);
-	int LookupThousands(int value, int thousandplex, char *ph_out);
-   int TranslateNumber_1(char *word1, char *ph_out, unsigned int *flags, int wflags);
-	int TranslateRoman(char *word, char *ph_out);
-	int TranslateChar(char *ptr, int prev_in, unsigned int c, unsigned int next_in, int *insert);
-
-	void InitGroups(void);
-	void AppendPhonemes(char *string, int size, const char *ph);
-	char *DecodeRule(const char *group, char *rule);
-	void MatchRule(char *word[], const char *group, char *rule, MatchRecord *match_out, int end_flags, int dict_flags);
-	int TranslateRules(char *p, char *phonemes, int size, char *end_phonemes, int end_flags, unsigned int *dict_flags);
-	void ApplySpecialAttribute(char *phonemes, int dict_flags);
-
-	int IsLetter(int letter, int group);
-	int IsLetterGroup(char *word, int group, int pre);
-
-	void CalcPitches_Tone(int clause_tone);
-
-protected:
-	virtual int Unpronouncable(char *word);
-	virtual void SetWordStress(char *output, unsigned int dictionary_flags, int tonic, int prev_stress);
-	virtual int RemoveEnding(char *word, int end_type, char *word_copy);
-   virtual int TranslateNumber(char *word1, char *ph_out, unsigned int *flags, int wflags);
-	virtual int ChangePhonemes(PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch);
-
-	int IsVowel(int letter);
-	int LookupDictList(char **wordptr, char *ph_out, unsigned int *flags, int end_flags, WORD_TAB *wtab);
-	int Lookup(const char *word, char *ph_out);
-
-	
 	// groups1 and groups2 are indexes into data_dictrules, set up by InitGroups()
 	// the two-letter rules for each letter must be consecutive in the language_rules source
 	
@@ -485,22 +445,13 @@ protected:
 	
 	unsigned char groups2_count[256];    // number of 2 letter groups for this initial letter
 	unsigned char groups2_start[256];    // index into groups2
-	char *letterGroups[N_LETTER_GROUPS];
 	
-	int n_ph_list2;
-	PHONEME_LIST2 ph_list2[N_PHONEME_LIST];	// first stage of text->phonemes
-
 	
-
 	int expect_verb;
 	int expect_past;    // expect past tense
 	int expect_verb_s;
 	int expect_noun;
-	int word_flags;     // word is all upper case
 	int prev_last_stress;
-	int prepause_timeout;
-	int end_stressed_vowel;  // word ends with stressed vowel
-	int prev_dict_flags;     // dictionary flags from previous word
 	char *clause_end;
 
 	int word_vowel_count;     // number of vowels so far
@@ -509,10 +460,12 @@ protected:
 	int clause_upper_count;   // number of upper case letters in the clause
 	int clause_lower_count;   // number of lower case letters in the clause
 
+	int prepause_timeout;
+	int end_stressed_vowel;  // word ends with stressed vowel
+	int prev_dict_flags;     // dictionary flags from previous word
 }; //  end of class Translator
 
 
-extern int option_tone1;
 extern int option_tone2;
 #define OPTION_EMPHASIZE_ALLCAPS  0x100
 #define OPTION_EMPHASIZE_PENULTIMATE 0x200
@@ -522,7 +475,6 @@ extern int option_quiet;
 extern int option_phonemes;
 extern int option_phoneme_events;
 extern int option_linelength;     // treat lines shorter than this as end-of-clause
-extern int option_harmonic1;
 extern int option_multibyte;
 extern int option_capitals;
 extern int option_punctuation;
@@ -553,9 +505,6 @@ extern char skip_marker[N_MARKER_LENGTH];
 #define N_PUNCTLIST  60
 extern wchar_t option_punctlist[N_PUNCTLIST];  // which punctuation characters to announce
 extern unsigned char punctuation_to_tone[INTONATION_TYPES][PUNCT_INTONATIONS];
-extern const unsigned short punct_chars[];   // punctuation chars fo end-of-clause
-
-extern int speech_parameters[];
 
 extern Translator *translator;
 extern Translator *translator2;
@@ -564,31 +513,22 @@ extern char dictionary_name[40];
 extern char ctrl_embedded;    // to allow an alternative CTRL for embedded commands
 extern char *p_textinput;
 extern wchar_t *p_wchar_input;
-extern int ungot_char;
 extern int dictionary_skipwords;
 
 extern int (* uri_callback)(int, const char *, const char *);
 extern int (* phoneme_callback)(const char *);
 extern void SetLengthMods(Translator *tr, int value);
 
-Translator *SelectTranslator(const char *name);
-int SetTranslator2(const char *name);
-int CompileDictionary(const char *dsource, const char *dict_name, FILE *log, char *err_name,int flags);
 void LoadConfig(void);
-int PhonemeCode(unsigned int mnem);
-void ChangeWordStress(Translator *tr, char *word, int new_stress);
 int TransposeAlphabet(char *text, int offset, int min, int max);
-int utf8_in(int *c, const char *buf, int backwards);
+int utf8_in(int *c, const char *buf);
+int utf8_in2(int *c, const char *buf, int backwards);
 int utf8_out(unsigned int c, char *buf);
 int utf8_nbytes(const char *buf);
 int lookupwchar(const unsigned short *list,int c);
 int Eof(void);
 char *strchr_w(const char *s, int c);
 int IsBracket(int c);
-void ResetLetterBits(Translator *tr, int groups);
-void SetLetterBits(Translator *tr, int group, const char *string);
-void SetLetterBitsRange(Translator *tr, int group, int first, int last);
-void SetLetterVowel(Translator *tr, int c);
 void InitNamedata(void);
 void InitText(int flags);
 void InitText2(void);
@@ -596,6 +536,41 @@ int IsDigit(unsigned int c);
 int IsAlpha(unsigned int c);
 int isspace2(unsigned int c);
 int towlower2(unsigned int c);
+void GetTranslatedPhonemeString(char *phon_out, int n_phon_out);
+
+Translator *SelectTranslator(const char *name);
+int SetTranslator2(const char *name);
+void DeleteTranslator(Translator *tr);
+int Lookup(Translator *tr, const char *word, char *ph_out);
+
+int TranslateNumber(Translator *tr, char *word1, char *ph_out, unsigned int *flags, int wflags);
+int TranslateRoman(Translator *tr, char *word, char *ph_out);
+
+void ChangeWordStress(Translator *tr, char *word, int new_stress);
+void SetSpellingStress(Translator *tr, char *phonemes, int control, int n_chars);
+int TranslateLetter(Translator *tr, char *letter, char *phonemes, int control, int word_length);
+void LookupLetter(Translator *tr, unsigned int letter, int next_byte, char *ph_buf);
+void LookupAccentedLetter(Translator *tr, unsigned int letter, char *ph_buf);
+
+int LoadDictionary(Translator *tr, const char *name, int no_error);
+int LookupDictList(Translator *tr, char **wordptr, char *ph_out, unsigned int *flags, int end_flags, WORD_TAB *wtab);
+
+void MakePhonemeList(Translator *tr, int post_pause, int new_sentence);
+int ChangePhonemes_ru(Translator *tr, PHONEME_LIST2 *phlist, int n_ph, int index, PHONEME_TAB *ph, CHANGEPH *ch);
+void ApplySpecialAttribute(Translator *tr, char *phonemes, int dict_flags);
+void AppendPhonemes(Translator *tr, char *string, int size, const char *ph);
+
+void CalcLengths(Translator *tr);
+void CalcPitches(Translator *tr, int clause_tone);
+
+int RemoveEnding(Translator *tr, char *word, int end_type, char *word_copy);
+int Unpronouncable(Translator *tr, char *word);
+void SetWordStress(Translator *tr, char *output, unsigned int dictionary_flags, int tonic, int prev_stress);
+int TranslateRules(Translator *tr, char *p, char *phonemes, int size, char *end_phonemes, int end_flags, unsigned int *dict_flags);
+int TranslateWord(Translator *tr, char *word1, int next_pause, WORD_TAB *wtab);
+void *TranslateClause(Translator *tr, FILE *f_text, const void *vp_input, int *tone, char **voice_change);
+int ReadClause(Translator *tr, FILE *f_in, char *buf, short *charix, int n_buf, int *tone_type);
+
 void SetVoiceStack(espeak_VOICE *v);
 
 extern FILE *f_trans;		// for logging
